@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-close-confirmation-dialog.c,v 1.18 2006/08/16 17:45:11 pborelli Exp $
+ * $Id: gedit-close-confirmation-dialog.c,v 1.21 2006/12/07 23:33:28 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -162,8 +162,22 @@ set_logout_mode (GeditCloseConfirmationDialog *dlg,
 	}
 	else
 	{
+		const gchar *stock_id = GTK_STOCK_SAVE;
+		
+		if (GET_MODE (dlg->priv) == SINGLE_DOC_MODE)
+		{
+			GeditDocument *doc;
+			
+			doc = GEDIT_DOCUMENT (dlg->priv->unsaved_documents->data);
+			
+			if (gedit_document_get_readonly (doc) || 
+			    gedit_document_is_untitled (doc))
+				stock_id = GTK_STOCK_SAVE_AS;
+		}
+
 		gtk_dialog_add_button (GTK_DIALOG (dlg),
-				       GTK_STOCK_SAVE, GTK_RESPONSE_YES);
+				       stock_id, 
+				       GTK_RESPONSE_YES);
 
 		gtk_dialog_set_default_response	(GTK_DIALOG (dlg), 
 						 GTK_RESPONSE_YES);
@@ -185,7 +199,8 @@ gedit_close_confirmation_dialog_init (GeditCloseConfirmationDialog *dlg)
 	gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dlg)->vbox), 14);
 	gtk_window_set_resizable (GTK_WINDOW (dlg), FALSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dlg), FALSE);
-
+	gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dlg), TRUE);
+	
 	gtk_window_set_title (GTK_WINDOW (dlg), "");
 
 	gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
@@ -454,7 +469,7 @@ get_text_secondary_label (GeditDocument *doc)
 		hours = seconds / 3600;
 
 		secondary_msg = g_strdup_printf (
-					ngettext ("If you don't save, changes from the last hour "
+					ngettext ("If you don't save, changes from the last %d hour "
 					    	  "will be permanently lost.",
 						  "If you don't save, changes from the last %d hours "
 					    	  "will be permanently lost.",
