@@ -27,7 +27,7 @@
  * list of people on the gedit Team.
  * See the ChangeLog files for a list of changes.
  *
- * $Id: gedit-commands-file.c,v 1.13 2006/08/18 19:17:38 pborelli Exp $
+ * $Id: gedit-commands-file.c,v 1.15 2006/12/07 23:33:28 paolo Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -902,9 +902,10 @@ file_save (GeditTab    *tab,
 	doc = gedit_tab_get_document (tab);
 	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
-	if (gedit_document_is_untitled (doc))
+	if (gedit_document_is_untitled (doc) || 
+	    gedit_document_get_readonly (doc))
 	{
-		gedit_debug_message (DEBUG_COMMANDS, "Untitled");
+		gedit_debug_message (DEBUG_COMMANDS, "Untitled or Readonly");
 
 		file_save_as (tab, window);
 		
@@ -1201,7 +1202,7 @@ revert_dialog (GeditWindow   *window,
 		hours = seconds / 3600;
 
 		secondary_msg = g_strdup_printf (
-					ngettext ("Changes made to the document in the last hour "
+					ngettext ("Changes made to the document in the last %d hour "
 					    	  "will be permanently lost.",
 						  "Changes made to the document in the last %d hours "
 					    	  "will be permanently lost.",
