@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document-loader.c,v 1.7 2006/04/20 17:08:08 pborelli Exp $
+ * $Id: gedit-document-loader.c,v 1.11 2006/12/01 12:07:55 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -257,6 +257,11 @@ insert_text_in_document (GeditDocumentLoader *loader,
 	gtk_source_buffer_begin_not_undoable_action (
 				GTK_SOURCE_BUFFER (loader->priv->document));
 
+	/* If the last char is a newline, don't add it to the buffer
+	(otherwise GtkTextView shows it as an empty line). See bug #324942. */
+	if (text[len-1] == '\n')
+		len--;
+
 	/* Insert text in the buffer */
 	gtk_text_buffer_set_text (GTK_TEXT_BUFFER (loader->priv->document), 
 				  text, 
@@ -275,7 +280,7 @@ update_document_contents (GeditDocumentLoader  *loader,
 			  gint                  file_size,
 			  GError              **error)
 {
-	gedit_debug (DEBUG_SAVER);
+	gedit_debug (DEBUG_LOADER);
 
 	g_return_val_if_fail (file_size > 0, FALSE);
 	g_return_val_if_fail (file_contents != NULL, FALSE);
@@ -515,7 +520,28 @@ load_local_file_real (GeditDocumentLoader *loader)
 
 		goto done;
 	}
-	
+
+	/* not a regular file */
+	if (!S_ISREG (statbuf.st_mode))
+	{
+		if (S_ISDIR (statbuf.st_mode))
+		{
+			g_set_error (&loader->priv->error,
+				     GEDIT_DOCUMENT_ERROR,
+				     GNOME_VFS_ERROR_IS_DIRECTORY,
+				     gnome_vfs_result_to_string (GNOME_VFS_ERROR_IS_DIRECTORY));
+		}
+		else
+		{
+			g_set_error (&loader->priv->error,
+				     GEDIT_DOCUMENT_ERROR,
+				     GEDIT_DOCUMENT_ERROR_NOT_REGULAR_FILE,
+				     "Not a regular file");
+		}
+
+		goto done;
+	}
+
 	loader->priv->info = gnome_vfs_file_info_new ();
 	stat_to_file_info (loader->priv->info, &statbuf);
 	GNOME_VFS_FILE_INFO_SET_LOCAL (loader->priv->info, TRUE);
@@ -543,7 +569,7 @@ load_local_file_real (GeditDocumentLoader *loader)
 				    
 		if (mapped_file == MAP_FAILED)
 		{
-			gedit_debug_message (DEBUG_SAVER, "mmap failed");
+			gedit_debug_message (DEBUG_LOADER, "mmap failed");
 
 			result = gnome_vfs_result_from_errno ();
 
@@ -618,7 +644,7 @@ static void
 load_local_file (GeditDocumentLoader *loader,
 		 const gchar         *fname)
 {
-	gedit_debug (DEBUG_SAVER);
+	gedit_debug (DEBUG_LOADER);
 
 	g_signal_emit (loader,
 		       signals[LOADING],
@@ -644,10 +670,10 @@ load_local_file (GeditDocumentLoader *loader,
 				    
 		return;			    
 	}
-	
+
 	g_free (loader->priv->local_file_name);
 	loader->priv->local_file_name = g_strdup (fname);
-	
+
 	g_timeout_add_full (G_PRIORITY_HIGH,
 			    0,
 			    (GSourceFunc) load_local_file_real,
@@ -734,10 +760,18 @@ async_read_cb (GnomeVFSAsyncHandle *handle,
 	/* end of the file, we are done! */
 	if (bytes_read == 0 || result != GNOME_VFS_OK)
 	{
-		update_document_contents (loader,
-					  loader->priv->buffer,
-					  loader->priv->bytes_read,
-					  &loader->priv->error);
+		if (loader->priv->bytes_read == 0)
+		{
+			if (loader->priv->encoding == NULL)
+				loader->priv->auto_detected_encoding = gedit_encoding_get_current ();
+		}
+		else
+		{
+			update_document_contents (loader,
+						  loader->priv->buffer,
+						  loader->priv->bytes_read,
+						  &loader->priv->error);
+		}
 
 		remote_load_completed_or_failed (loader);
 
