@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document-saver.c,v 1.12 2006/08/25 09:40:35 paolo Exp $
+ * $Id: gedit-document-saver.c,v 1.15 2006/11/26 13:13:14 sfre Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -191,7 +191,6 @@ write_document_contents (gint                  fd,
 	GtkTextIter end_iter;
 	gchar *contents;
 	gsize len;
-	gboolean add_cr;
 	ssize_t written;
 	gboolean res;
 
@@ -202,11 +201,6 @@ write_document_contents (gint                  fd,
 
 	len = strlen (contents);
 
-	if (len >= 1)
-		add_cr = (*(contents + len - 1) != '\n');
-	else
-		add_cr = FALSE;
-
 	if (encoding != gedit_encoding_get_utf8 ())
 	{
 		gchar *converted_contents;
@@ -241,12 +235,31 @@ write_document_contents (gint                  fd,
 	/* Save the file content */
 	if (res)
 	{
-		written = write (fd, contents, len);
-		res = ((written != -1) && ((gsize) written == len));
+		const gchar *write_buffer = contents;
+		ssize_t to_write = len;
+
+		do
+		{
+			written = write (fd, write_buffer, to_write);
+			if (written == -1)
+			{
+				if (errno == EINTR)
+					continue;
+
+				res = FALSE;
+
+				break;
+			}
+
+			to_write -= written;
+			write_buffer += written;
+		}
+		while (to_write > 0);
 	}
 
-	/* Add \n at the end if needed */
-	if (res && add_cr)
+	/* make sure files are always terminated with \n (see bug #95676). Note
+	that we strip the trailing \n when loading the file */
+	if (res)
 	{
 		if (encoding != gedit_encoding_get_utf8 ())
 		{
@@ -374,7 +387,6 @@ copy_file_data (gint     sfd,
 		GError **error)
 {
 	gboolean ret = TRUE;
-	GError *err = NULL;
 	gpointer buffer;
 	const gchar *write_buffer;
 	ssize_t bytes_read;
@@ -392,7 +404,7 @@ copy_file_data (gint     sfd,
 		{
 			GnomeVFSResult result = gnome_vfs_result_from_errno ();
 
-			g_set_error (&err,
+			g_set_error (error,
 				     GEDIT_DOCUMENT_ERROR,
 				     result,
 				     gnome_vfs_result_to_string (result));
@@ -410,9 +422,14 @@ copy_file_data (gint     sfd,
 			bytes_written = write (dfd, write_buffer, bytes_to_write);
 			if (bytes_written == -1)
 			{
-				GnomeVFSResult result = gnome_vfs_result_from_errno ();
+				GnomeVFSResult result;
 
-				g_set_error (&err,
+				if (errno == EINTR)
+					continue;
+
+				result = gnome_vfs_result_from_errno ();
+
+				g_set_error (error,
 					     GEDIT_DOCUMENT_ERROR,
 					     result,
 					     gnome_vfs_result_to_string (result));
@@ -429,8 +446,7 @@ copy_file_data (gint     sfd,
 
 	} while ((bytes_read != 0) && (ret == TRUE));
 
-	if (error)
-		*error = err;
+	g_free (buffer);
 
 	return ret;
 }
@@ -1270,11 +1286,11 @@ async_xfer_progress (GnomeVFSAsyncHandle      *handle,
 		     GnomeVFSXferProgressInfo *progress_info,
 		     gpointer                  data)
 {
+	GeditDocumentSaver *saver = GEDIT_DOCUMENT_SAVER (data);
+
 	gedit_debug_message (DEBUG_SAVER, "xfer phase: %d", progress_info->phase);
 	gedit_debug_message (DEBUG_SAVER, "xfer status: %d", progress_info->status);
 	
-	GeditDocumentSaver *saver = GEDIT_DOCUMENT_SAVER (data);
-
 	switch (progress_info->status)
 	{
 	case GNOME_VFS_XFER_PROGRESS_STATUS_OK:
