@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-io-error-message-area.c,v 1.7 2006/08/17 19:55:29 pborelli Exp $
+ * $Id: gedit-io-error-message-area.c,v 1.10 2006/12/18 21:10:05 pborelli Exp $
  */
  
 /*
@@ -52,9 +52,32 @@
 
 #define MAX_URI_IN_DIALOG_LENGTH 50
 
+static gboolean
+is_recoverable_error (const GError *error)
+{
+	gboolean is_recoverable = FALSE;
+
+	if (error->domain == GEDIT_DOCUMENT_ERROR)
+	{
+		switch (error->code) {
+		case GNOME_VFS_ERROR_ACCESS_DENIED:
+		case GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES:
+		case GNOME_VFS_ERROR_NO_MEMORY:
+		case GNOME_VFS_ERROR_HOST_NOT_FOUND:
+		case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
+		case GNOME_VFS_ERROR_LOGIN_FAILED:
+		case GNOME_VFS_ERROR_TIMEOUT:
+
+			is_recoverable = TRUE;
+		}
+	}
+
+	return is_recoverable;
+}
+
 static GtkWidget *
-create_unrecoverable_error_message_area (const gchar *primary_text,
-					 const gchar *secondary_text)
+create_io_loading_error_message_area (const gchar *primary_text,
+				      const gchar *secondary_text)
 {
 	GtkWidget *message_area;
 	GtkWidget *hbox_content;
@@ -113,8 +136,8 @@ create_unrecoverable_error_message_area (const gchar *primary_text,
 }
 
 GtkWidget *
-gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
-				    		    const GError *error)
+gedit_io_loading_error_message_area_new (const gchar  *uri,
+					 const GError *error)
 {
 	gchar *error_message = NULL;
 	gchar *message_details = NULL;
@@ -147,7 +170,7 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 	case GNOME_VFS_ERROR_NOT_FOUND:
 		error_message = g_strdup_printf (_("Could not find the file %s."),
 						 uri_for_display);
-		message_details = g_strdup (_("Please, check that you typed the "
+		message_details = g_strdup (_("Please check that you typed the "
 				      	      "location correctly and try again."));
 	break;
 
@@ -186,7 +209,7 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 	case GNOME_VFS_ERROR_INVALID_URI:
 		error_message = g_strdup_printf (_("%s is not a valid location."),
 						 uri_for_display);
-		message_details = g_strdup (_("Please, check that you typed the "
+		message_details = g_strdup (_("Please check that you typed the "
 					      "location correctly and try again."));
 		break;
 
@@ -195,20 +218,20 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 		break;
 
 	case GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES:
-		message_details = g_strdup (_("There are too many open files. Please, "
+		message_details = g_strdup (_("There are too many open files. Please "
 					      "close some applications and try again."));
 		break;
 
 	case GNOME_VFS_ERROR_IS_DIRECTORY:
 		error_message = g_strdup_printf (_("%s is a directory."),
 						 uri_for_display);
-		message_details = g_strdup (_("Please, check that you typed the "
+		message_details = g_strdup (_("Please check that you typed the "
 					      "location correctly and try again."));
 		break;
 
 	case GNOME_VFS_ERROR_NO_MEMORY:
 		message_details = g_strdup (_("Not enough available memory to open "
-					      "the file. Please, close some running "
+					      "the file. Please close some running "
 					      "applications and try again."));
 		break;
 
@@ -236,7 +259,7 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 					/* Translators: %s is a host name */
 					message_details = g_strdup_printf (
 						_("Host %s could not be found. "
-	        		  	  	  "Please, check that your proxy settings "
+	        		  	  	  "Please check that your proxy settings "
 				  	  	  "are correct and try again."),
 					  	host_markup);
 
@@ -247,7 +270,7 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 					/* use the same string as INVALID_HOST */
 					message_details = g_strdup_printf (
 						_("Host name was invalid. "
-						  "Please, check that you typed the location "
+						  "Please check that you typed the location "
 						  "correctly and try again."));
 				}
 
@@ -258,7 +281,7 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 				/* use the same string as INVALID_HOST */
 				message_details = g_strdup_printf (
 					_("Host name was invalid. "
-					  "Please, check that you typed the location "
+					  "Please check that you typed the location "
 					  "correctly and try again."));
 			}
 		}
@@ -266,19 +289,19 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 
 	case GNOME_VFS_ERROR_INVALID_HOST_NAME:
 		message_details = g_strdup_printf (_("Host name was invalid. "
-						     "Please, check that you typed the location "
+						     "Please check that you typed the location "
 						     "correctly and try again."));
         		break;
 
 	case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
 		message_details = g_strdup (_("Host name was empty. "
-					      "Please, check that your proxy settings "
+					      "Please check that your proxy settings "
 					      "are correct and try again."));
 		break;
 
 	case GNOME_VFS_ERROR_LOGIN_FAILED:
 		message_details = g_strdup (_("Attempt to log in failed. "
-					      "Please, check that you typed the location "
+					      "Please check that you typed the location "
 					      "correctly and try again."));
 		break;
 
@@ -286,6 +309,10 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 		message_details = g_strdup (_("The file you are trying to open is not a regular file."));
 		break;
 
+	case GNOME_VFS_ERROR_TIMEOUT:
+		message_details = g_strdup (_("Connection timed out. Please try again."));
+		break;
+
 	case GNOME_VFS_ERROR_GENERIC:
 		break;
 
@@ -331,8 +358,16 @@ gedit_unrecoverable_loading_error_message_area_new (const gchar  *uri,
 		error_message = g_strdup_printf (_("Could not open the file %s."),
 						 uri_for_display);
 
-	message_area = create_unrecoverable_error_message_area (error_message,
-								message_details);
+	message_area = create_io_loading_error_message_area (error_message,
+							     message_details);
+
+	if (is_recoverable_error (error))
+	{
+		gedit_message_area_add_stock_button_with_text (GEDIT_MESSAGE_AREA (message_area),
+							       _("_Retry"),
+							       GTK_STOCK_REFRESH,
+							       GTK_RESPONSE_OK);
+	}
 
 	g_free (uri_for_display);
 	g_free (error_message);
@@ -375,7 +410,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 	{
 	case GNOME_VFS_ERROR_NOT_FOUND:
 		message_details = g_strdup (_("gedit cannot find it. "
-					      "Perhaps, it has recently been deleted."));
+					      "Perhaps it has recently been deleted."));
 		break;
 
 	case GNOME_VFS_ERROR_CORRUPTED_DATA:
@@ -415,7 +450,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 		break;
 
 	case GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES:
-		message_details = g_strdup (_("There are too many open files. Please, "
+		message_details = g_strdup (_("There are too many open files. Please "
 					      "close some applications and try again."));
 		break;
 
@@ -426,7 +461,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 
 	case GNOME_VFS_ERROR_NO_MEMORY:
 		message_details = g_strdup (_("Not enough available memory to open "
-					      "the file. Please, close some running "
+					      "the file. Please close some running "
 					      "applications and try again."));
 		break;
 
@@ -454,7 +489,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 					/* Translators: %s is a host name */
 					message_details = g_strdup_printf (
 						_("Host %s could not be found. "
-	        		  	  	"Please, check that your proxy settings "
+	        		  	  	"Please check that your proxy settings "
 				  	  	"are correct and try again."),
 					  	host_markup);
 
@@ -465,7 +500,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 					/* use the same string as INVALID_HOST */
 					message_details = g_strdup_printf (
 						_("Host name was invalid. "
-						  "Please, check that you typed the location "
+						  "Please check that you typed the location "
 						  "correctly and try again."));
 				}
 
@@ -476,7 +511,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 				/* use the same string as INVALID_HOST */
 				message_details = g_strdup_printf (
 					_("Host name was invalid. "
-					  "Please, check that you typed the location "
+					  "Please check that you typed the location "
 					  "correctly and try again."));
 			}
 		}
@@ -484,7 +519,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 
 	case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
 		message_details = g_strdup (_("Host name was empty. "
-					      "Please, check that your proxy settings "
+					      "Please check that your proxy settings "
 					      "are correct and try again."));
 		break;
 
@@ -497,6 +532,10 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 						   uri_for_display);
 		break;
 
+	case GNOME_VFS_ERROR_TIMEOUT:
+		message_details = g_strdup (_("Connection timed out. Please try again."));
+		break;
+
 	/* this should never happen, revert should be insensitive */
 //CHECK: we used to have this before new_mdi... is it really useful (see comment above)
 /*
@@ -504,6 +543,7 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 		message_details = g_strdup (_("It is not possible to revert an Untitled document."));
 		break;
 */
+
 	case GNOME_VFS_ERROR_GENERIC:
 		break;
 
@@ -552,8 +592,8 @@ gedit_unrecoverable_reverting_error_message_area_new (const gchar  *uri,
 		error_message = g_strdup_printf (_("Could not revert the file %s."),
 						 uri_for_display);
 
-	message_area = create_unrecoverable_error_message_area (error_message,
-								message_details);
+	message_area = create_io_loading_error_message_area (error_message,
+							     message_details);
 
 	g_free (uri_for_display);
 	g_free (error_message);
@@ -854,7 +894,7 @@ gedit_file_already_open_warning_message_area_new (const gchar *uri)
 	gtk_widget_show (vbox);
 	gtk_box_pack_start (GTK_BOX (hbox_content), vbox, TRUE, TRUE, 0);
 
-	primary_text = g_strdup_printf (_("This file (%s) is already opened in another gedit window."), uri_for_display);
+	primary_text = g_strdup_printf (_("This file (%s) is already open in another gedit window."), uri_for_display);
 	g_free (uri_for_display);
 	
 	primary_markup = g_strdup_printf ("<b>%s</b>", primary_text);
@@ -869,7 +909,7 @@ gedit_file_already_open_warning_message_area_new (const gchar *uri)
 	GTK_WIDGET_SET_FLAGS (primary_label, GTK_CAN_FOCUS);
 	gtk_label_set_selectable (GTK_LABEL (primary_label), TRUE);
 
-	secondary_text = _("gedit opened this instance of the file in non-editable way. "
+	secondary_text = _("gedit opened this instance of the file in a non-editable way. "
 			   "Do you want to edit it anyway?");
   	secondary_markup = g_strdup_printf ("<small>%s</small>",
   					    secondary_text);
@@ -1126,15 +1166,15 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
  
 				/* Translators: %s is a URI scheme (like for example http:, ftp:, etc.) */
 				message_details = g_strdup_printf (_("gedit cannot handle %s locations in write mode. "
-								     "Please, check that you typed the "
+								     "Please check that you typed the "
 								     "location correctly and try again."),
 								   scheme_markup);
 				g_free (scheme_markup);
 			}
 			else
 			{
-				message_details = g_strdup (_("gedit cannot handle this location  in write mode. "
-							      "Please, check that you typed the "
+				message_details = g_strdup (_("gedit cannot handle this location in write mode. "
+							      "Please check that you typed the "
 							      "location correctly and try again."));
 			}
 
@@ -1150,32 +1190,32 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 
 		case GNOME_VFS_ERROR_INVALID_URI:
 			message_details = g_strdup (_("%s is not a valid location. " 
-						      "Please, check that you typed the "
+						      "Please check that you typed the "
 						      "location correctly and try again."));
 			break;
 
 		case GNOME_VFS_ERROR_ACCESS_DENIED:
 		case GNOME_VFS_ERROR_NOT_PERMITTED:		
 			message_details = g_strdup (_("You do not have the permissions necessary to save the file. "
-						      "Please, check that you typed the "
+						      "Please check that you typed the "
 						      "location correctly and try again."));
 
 			break;
 
 		case GNOME_VFS_ERROR_TOO_MANY_OPEN_FILES:
-			message_details = g_strdup (_("There are too many open files. Please, "
+			message_details = g_strdup (_("There are too many open files. Please "
 						      "close some applications and try again."));
 			break;
 
 		case GNOME_VFS_ERROR_IS_DIRECTORY:
 			message_details = g_strdup (_("%s is a directory. "
-						      "Please, check that you typed the "
+						      "Please check that you typed the "
 						      "location correctly and try again."));
 			break;
 
 		case GNOME_VFS_ERROR_NO_MEMORY:
 			message_details = g_strdup (_("Not enough available memory to save "
-						      "the file. Please, close some running "
+						      "the file. Please close some running "
 						      "applications and try again."));
 			break;
 
@@ -1203,7 +1243,7 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 						/* Translators: %s is a host name */
 						message_details = g_strdup_printf (
 							_("Host %s could not be found. "
-		        		  	  	  "Please, check that your proxy settings "
+		        		  	  	  "Please check that your proxy settings "
 					  	  	  "are correct and try again."),
 						  	host_markup);
 
@@ -1214,7 +1254,7 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 						/* use the same string as INVALID_HOST */
 						message_details = g_strdup_printf (
 							_("Host name was invalid. "
-							  "Please, check that you typed the location "
+							  "Please check that you typed the location "
 							  "correctly and try again."));
 					}
 
@@ -1225,7 +1265,7 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 					/* use the same string as INVALID_HOST */
 					message_details = g_strdup_printf (
 						_("Host name was invalid. "
-						  "Please, check that you typed the location "
+						  "Please check that you typed the location "
 						  "correctly and try again."));
 				}
 			}
@@ -1233,47 +1273,51 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 
 		case GNOME_VFS_ERROR_INVALID_HOST_NAME:
 			message_details = g_strdup_printf (_("Host name was invalid. "
-							     "Please, check that you typed the location "
+							     "Please check that you typed the location "
 							     "correctly and try again."));
 				break;
 
 		case GNOME_VFS_ERROR_HOST_HAS_NO_ADDRESS:
 			message_details = g_strdup (_("Host name was empty. "
-						      "Please, check that your proxy settings "
+						      "Please check that your proxy settings "
 						      "are correct and try again."));
 			break;
 
 		case GNOME_VFS_ERROR_LOGIN_FAILED:
 			message_details = g_strdup (_("Attempt to log in failed. "
-						      "Please, check that you typed the location "
+						      "Please check that you typed the location "
 						      "correctly and try again."));
 			break;
 
 		case GNOME_VFS_ERROR_NO_SPACE:
 			message_details = g_strdup (_("There is not enough disk space to save the file. "
-						      "Please, free some disk space and try again."));
+						      "Please free some disk space and try again."));
 			break;
 			
 		case GNOME_VFS_ERROR_READ_ONLY:
 		case GNOME_VFS_ERROR_READ_ONLY_FILE_SYSTEM:		
 			message_details = g_strdup (_("You are trying to save the file on a read-only disk. "
-						      "Please, check that you typed the location "
+						      "Please check that you typed the location "
 						      "correctly and try again."));
 			break;
 			
 		case GNOME_VFS_ERROR_FILE_EXISTS:
 			message_details = g_strdup (_("A file with the same name already exists. "
-						      "Please, use a different name."));
+						      "Please use a different name."));
 			break;
 			
 		case GNOME_VFS_ERROR_NAME_TOO_LONG:
 			message_details = g_strdup (_("The disk where you are trying to save the file has "
 						      "a limitation on length of the file names. "
-						      "Please, use a shorter name."));
+						      "Please use a shorter name."));
+
+		case GNOME_VFS_ERROR_TIMEOUT:
+			message_details = g_strdup (_("Connection timed out. Please try again."));
+			break;
 
 		case GEDIT_DOCUMENT_ERROR_NOT_REGULAR_FILE:
 			message_details = g_strdup_printf (_("%s is not a regular file. "
-							     "Please, check that you typed the location "
+							     "Please check that you typed the location "
 							     "correctly and try again."),
 							   uri_for_display);
 
@@ -1318,8 +1362,8 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 		error_message = g_strdup_printf (_("Could not save the file %s."),
 						 uri_for_display);
 
-	message_area = create_unrecoverable_error_message_area (error_message,
-								message_details);
+	message_area = create_io_loading_error_message_area (error_message,
+							     message_details);
 
 	g_free (uri_for_display);
 	g_free (error_message);
@@ -1327,3 +1371,4 @@ gedit_unrecoverable_saving_error_message_area_new (const gchar  *uri,
 
 	return message_area;
 }
+
