@@ -32,7 +32,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-spinner.c,v 1.4 2006/10/01 13:19:28 pborelli Exp $
+ * $Id: gedit-spinner.c,v 1.6 2006/12/09 19:47:18 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -86,14 +86,13 @@ typedef struct
 	GtkIconSize  size;
 	gint         width;
 	gint         height;
-	GdkPixbuf   *quiescent_pixbuf;
 	GdkPixbuf  **animation_pixbufs;
 	guint        n_animation_pixbufs;
 } GeditSpinnerImages;
 
 #define LAST_ICON_SIZE			GTK_ICON_SIZE_DIALOG + 1
-#define SPINNER_ICON_NAME		"gnome-spinner"
-#define SPINNER_REST_ICON_NAME		"gnome-spinner-rest"
+#define SPINNER_ICON_NAME		"process-working"
+#define SPINNER_FALLBACK_ICON_NAME	"gnome-spinner"
 #define GEDIT_SPINNER_IMAGES_INVALID	((GeditSpinnerImages *) 0x1)
 
 typedef struct
@@ -164,8 +163,6 @@ gedit_spinner_images_unref (GeditSpinnerImages *images)
 		}
 		g_free (images->animation_pixbufs);
 
-		g_object_unref (images->quiescent_pixbuf);
-
 		g_free (images);
 	}
 }
@@ -244,7 +241,6 @@ gedit_spinner_images_load (GdkScreen *screen,
 			   GtkIconSize icon_size)
 {
 	GeditSpinnerImages *images;
-	GdkPixbuf *rest_pixbuf = NULL;
 	GdkPixbuf *icon_pixbuf, *pixbuf;
 	GtkIconInfo *icon_info = NULL;
 	int grid_width, grid_height, x, y, requested_size, size, isw, ish, n;
@@ -261,44 +257,26 @@ gedit_spinner_images_load (GdkScreen *screen,
  
 	requested_size = MAX (ish, isw);
 
-	/* Load the rest icon */
+	/* Load the animation. The 'rest icon' is the 0th frame */
 	icon_info = gtk_icon_theme_lookup_icon (icon_theme,
-						SPINNER_REST_ICON_NAME,
+						SPINNER_ICON_NAME,
 						requested_size, 0);
 	if (icon_info == NULL)
-	{
-		g_warning ("Throbber rest icon not found");
-		goto loser;
-	}
-
-	size = gtk_icon_info_get_base_size (icon_info);
-	icon = gtk_icon_info_get_filename (icon_info);
-	if (icon == NULL)
-		goto loser;
-
-	rest_pixbuf = gdk_pixbuf_new_from_file (icon, NULL);
-	gtk_icon_info_free (icon_info);
-
-	if (rest_pixbuf == NULL)
-	{
-		g_warning ("Could not load spinner rest icon");
-		goto loser;
-	}
-
-	if (size > requested_size)
-	{
-		rest_pixbuf = scale_to_size (rest_pixbuf, isw, ish);
-	}
-
-	/* Load the animation */
-	icon_info = gtk_icon_theme_lookup_icon (icon_theme,
-						SPINNER_ICON_NAME,
-					        requested_size, 0);
-	if (icon_info == NULL)
 	{
 		g_warning ("Throbber animation not found");
-		goto loser;
- 	}
+
+		/* If the icon naming spec compliant name wasn't found, try the old name */
+		icon_info = gtk_icon_theme_lookup_icon (icon_theme,
+							SPINNER_FALLBACK_ICON_NAME,
+						        requested_size, 0);
+		if (icon_info == NULL)
+		{
+			g_warning ("Throbber fallback animation not found either");
+			goto loser;
+	 	}
+	}
+
+	g_assert (icon_info != NULL);
 
 	size = gtk_icon_info_get_base_size (icon_info);
 	icon = gtk_icon_info_get_filename (icon_info);
@@ -308,6 +286,7 @@ gedit_spinner_images_load (GdkScreen *screen,
 
 	icon_pixbuf = gdk_pixbuf_new_from_file (icon, NULL);
 	gtk_icon_info_free (icon_info);
+	icon_info = NULL;
 
 	if (icon_pixbuf == NULL)
 	{
@@ -358,16 +337,16 @@ gedit_spinner_images_load (GdkScreen *screen,
  
 	images->size = icon_size;
 	images->width = images->height = requested_size;
-	images->quiescent_pixbuf = rest_pixbuf;
 
 	images->n_animation_pixbufs = n;
 	images->animation_pixbufs = g_new (GdkPixbuf *, n);
 
 	for (l = list; l != NULL; l = l->next)
 	{
+		g_assert (l->data != NULL);
 		images->animation_pixbufs[--n] = l->data;
 	}
-	/* g_assert (n == 0); */
+	g_assert (n == 0);
 
 	g_slist_free (list);
 
@@ -379,10 +358,7 @@ loser:
 	{
 		gtk_icon_info_free (icon_info);
  	}
-	if (rest_pixbuf)
-	{
-		g_object_unref (rest_pixbuf);
-	}
+
 	g_slist_foreach (list, (GFunc) g_object_unref, NULL);
 
 	/* STOP_PROFILER ("loading spinner animation") */
@@ -442,8 +418,10 @@ gedit_spinner_cache_get_images (GeditSpinnerCache *cache,
 	}
 
 	data = g_hash_table_lookup (priv->hash, screen);
+	if (data == NULL)
 	{
 		data = gedit_spinner_cache_data_new (screen);
+		/* FIXME: think about what happens when the screen's display is closed later on */
 		g_hash_table_insert (priv->hash, screen, data);
 	}
 
@@ -557,7 +535,7 @@ struct _GeditSpinnerPrivate
 static void gedit_spinner_class_init	(GeditSpinnerClass *class);
 static void gedit_spinner_init		(GeditSpinner      *spinner);
 
-static GObjectClass *parent_class = NULL;
+static GObjectClass *parent_class;
 
 GType
 gedit_spinner_get_type (void)
@@ -579,7 +557,7 @@ gedit_spinner_get_type (void)
 			(GInstanceInitFunc) gedit_spinner_init
 		};
 
-		type = g_type_register_static (GTK_TYPE_EVENT_BOX,
+		type = g_type_register_static (GTK_TYPE_WIDGET,
 					       "GeditSpinner",
 					       &our_info, 0);
 	}
@@ -603,7 +581,7 @@ gedit_spinner_load_images (GeditSpinner *spinner)
 
 		/* STOP_PROFILER ("gedit_spinner_load_images") */
 
-		priv->current_image = 0;
+		priv->current_image = 0; /* 'rest' icon */
 		priv->need_load = FALSE;
 	}
 
@@ -640,6 +618,8 @@ gedit_spinner_init (GeditSpinner *spinner)
 
 	priv = spinner->priv = GEDIT_SPINNER_GET_PRIVATE (spinner);
 
+	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (spinner), GTK_NO_WINDOW);
+
 	priv->cache = gedit_spinner_cache_ref ();
 	priv->size = GTK_ICON_SIZE_DIALOG;
 	priv->spinning = FALSE;
@@ -676,23 +656,15 @@ gedit_spinner_expose (GtkWidget      *widget,
 		return FALSE;
 	}
 
-	if (priv->spinning &&
-	    images->n_animation_pixbufs > 0)
-	{
-		g_assert (priv->current_image >= 0 &&
-			  priv->current_image < images->n_animation_pixbufs);
+	/* Otherwise |images| will be NULL anyway */
+	g_assert (images->n_animation_pixbufs > 0);
 
-		pixbuf = images->animation_pixbufs[priv->current_image];
-	}
-	else
-	{
-		pixbuf = images->quiescent_pixbuf;
-	}
+	g_assert (priv->current_image >= 0 &&
+		  priv->current_image < images->n_animation_pixbufs);
 
-	if (pixbuf == NULL)
-	{
-		return FALSE;
-	}
+	pixbuf = images->animation_pixbufs[priv->current_image];
+
+	g_assert (pixbuf != NULL);
 
 	width = gdk_pixbuf_get_width (pixbuf);
 	height = gdk_pixbuf_get_height (pixbuf);
@@ -738,7 +710,8 @@ bump_spinner_frame_cb (GeditSpinner *spinner)
 	priv->current_image++;
 	if (priv->current_image >= priv->images->n_animation_pixbufs)
 	{
-		priv->current_image = 0;
+		/* the 0th frame is the 'rest' icon */
+		priv->current_image = MIN (1, priv->images->n_animation_pixbufs);
 	}
 
 	gtk_widget_queue_draw (GTK_WIDGET (spinner));
@@ -764,11 +737,14 @@ gedit_spinner_start (GeditSpinner *spinner)
 			       priv->timer_task == 0 &&
 			       gedit_spinner_load_images (spinner))
 	{
-		priv->current_image = 0;
+		/* the 0th frame is the 'rest' icon */
+		priv->current_image = MIN (1, priv->images->n_animation_pixbufs);
 
-		priv->timer_task = g_timeout_add (priv->timeout,
-						  (GSourceFunc) bump_spinner_frame_cb,
-						  spinner);
+		priv->timer_task = g_timeout_add_full (G_PRIORITY_LOW,
+						       priv->timeout,
+						       (GSourceFunc) bump_spinner_frame_cb,
+						       spinner,
+						       NULL);
 	}
 }
 
@@ -796,6 +772,7 @@ gedit_spinner_stop (GeditSpinner *spinner)
 	GeditSpinnerPrivate *priv = spinner->priv;
 
 	priv->spinning = FALSE;
+	priv->current_image = 0;
 
 	if (priv->timer_task != 0)
 	{
@@ -1010,7 +987,5 @@ gedit_spinner_class_init (GeditSpinnerClass *class)
 GtkWidget *
 gedit_spinner_new (void)
 {
-	return GTK_WIDGET (g_object_new (GEDIT_TYPE_SPINNER,
-					 "visible-window", FALSE,
-					 NULL));
+	return GTK_WIDGET (g_object_new (GEDIT_TYPE_SPINNER, NULL));
 }
