@@ -388,7 +388,7 @@ gedit_tab_set_state (GeditTab      *tab,
 
 	set_view_properties_according_to_state (tab, state);
 
-	if ((state == GEDIT_TAB_STATE_LOADING_ERROR) || // FIXME: add other states if needed
+	if ((state == GEDIT_TAB_STATE_LOADING_ERROR) || /* FIXME: add other states if needed */
 	    (state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW))
 	{
 		gtk_widget_hide (tab->priv->view_scrolled_window);
@@ -461,17 +461,42 @@ remove_tab (GeditTab *tab)
 }
 
 static void 
-unrecoverable_loading_error_message_area_response (GeditMessageArea *message_area,
-						   gint              response_id,
-						   GeditTab         *tab)
+io_loading_error_message_area_response (GeditMessageArea *message_area,
+					gint              response_id,
+					GeditTab         *tab)
 {
-	remove_tab (tab);
+	if (response_id == GTK_RESPONSE_OK)
+	{
+		GeditDocument *doc;
+		gchar *uri;
+
+		doc = gedit_tab_get_document (tab);
+		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+
+		uri = gedit_document_get_uri (doc);
+		g_return_if_fail (uri != NULL);
+
+		set_message_area (tab, NULL);
+		gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
+
+		g_return_if_fail (tab->priv->auto_save_timeout <= 0);
+
+		gedit_document_load (doc,
+				     uri,
+				     tab->priv->tmp_encoding,
+				     tab->priv->tmp_line_pos,
+				     FALSE);
+	}
+	else
+	{
+		remove_tab (tab);
+	}
 }
 
 static void 
-recoverable_loading_error_message_area_response (GeditMessageArea *message_area,
-						 gint              response_id,
-						 GeditTab         *tab)
+conversion_loading_error_message_area_response (GeditMessageArea *message_area,
+						gint              response_id,
+						GeditTab         *tab)
 {
 	GeditDocument *doc;
 	gchar *uri;
@@ -485,17 +510,17 @@ recoverable_loading_error_message_area_response (GeditMessageArea *message_area,
 	if (response_id == GTK_RESPONSE_OK)
 	{
 		const GeditEncoding *encoding;
-		
+
 		encoding = gedit_conversion_error_message_area_get_encoding (
 				GTK_WIDGET (message_area));
 
 		g_return_if_fail (encoding != NULL);
-			
+
 		set_message_area (tab, NULL);
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
-	
+
 		tab->priv->tmp_encoding = encoding;
-		
+
 		g_return_if_fail (tab->priv->auto_save_timeout <= 0);
 
 		gedit_document_load (doc,
@@ -508,9 +533,7 @@ recoverable_loading_error_message_area_response (GeditMessageArea *message_area,
 	{
 		_gedit_recent_remove (GEDIT_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))), uri);
 
-		unrecoverable_loading_error_message_area_response (message_area,
-								   response_id,
-								   tab);
+		remove_tab (tab);
 	}
 
 	g_free (uri);
@@ -576,41 +599,40 @@ show_loading_message_area (GeditTab *tab)
 {
 	GtkWidget *area;
 	GeditDocument *doc = NULL;
-	const gchar *short_name;
 	gchar *name;
 	gchar *dirname = NULL;
 	gchar *msg = NULL;
 	gchar *name_markup;
 	gchar *dirname_markup;
 	gint len;
-	
+
 	if (tab->priv->message_area != NULL)
 		return;
-	
+
 	gedit_debug (DEBUG_TAB);
 		
 	doc = gedit_tab_get_document (tab);
 	g_return_if_fail (doc != NULL);
 
-	short_name = gedit_document_get_short_name_for_display (doc);
-
-	len = g_utf8_strlen (short_name, -1);
+	name = gedit_document_get_short_name_for_display (doc);
+	len = g_utf8_strlen (name, -1);
 
 	/* if the name is awfully long, truncate it and be done with it,
 	 * otherwise also show the directory (ellipsized if needed)
 	 */
 	if (len > MAX_MSG_LENGTH)
 	{
-		name = gedit_utils_str_middle_truncate (short_name, 
-							MAX_MSG_LENGTH);
+		gchar *str;
+
+		str = gedit_utils_str_middle_truncate (name, MAX_MSG_LENGTH);
+		g_free (name);
+		name = str;
 	}
 	else
 	{
 		gchar *uri;
 		gchar *str;
 
-		name = g_strdup (short_name);
-
 		uri = gedit_document_get_uri_for_display (doc);
 		str = gedit_utils_uri_get_dirname (uri);
 		g_free (uri);
@@ -925,11 +947,11 @@ document_loaded (GeditDocument *document,
 
 				if (tab->priv->state == GEDIT_TAB_STATE_LOADING_ERROR)
 				{
-					emsg = gedit_unrecoverable_loading_error_message_area_new (uri, 
-												   error);
+					emsg = gedit_io_loading_error_message_area_new (uri, 
+											error);
 					g_signal_connect (emsg,
 							  "response",
-							  G_CALLBACK (unrecoverable_loading_error_message_area_response),
+							  G_CALLBACK (io_loading_error_message_area_response),
 							  tab);
 				}
 				else
@@ -964,7 +986,7 @@ document_loaded (GeditDocument *document,
 
 			g_signal_connect (emsg,
 					  "response",
-					  G_CALLBACK (recoverable_loading_error_message_area_response),
+					  G_CALLBACK (conversion_loading_error_message_area_response),
 					  tab);
 		}
 
@@ -1973,9 +1995,18 @@ print_preview_destroyed (GtkWidget *preview,
 	tab->priv->print_preview = NULL;
 	
 	if (tab->priv->state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW)
+	{
 		gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
+	}
 	else
-		gtk_widget_show (tab->priv->view_scrolled_window);		
+	{
+		/* This should happen only when printing while showing the print
+		 * preview. In this case let us continue whithout changing
+		 * the state and show the document. See bug #352658 */
+		gtk_widget_show (tab->priv->view_scrolled_window);
+		
+		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
+	}	
 }
 
 static void
@@ -1986,8 +2017,7 @@ set_print_preview (GeditTab  *tab, GtkWidget *print_preview)
 		
 	if (tab->priv->print_preview != NULL)
 		gtk_widget_destroy (tab->priv->print_preview);
-		
-	
+
 	tab->priv->print_preview = print_preview;
 
 	gtk_box_pack_end (GTK_BOX (tab),
@@ -2072,8 +2102,16 @@ print_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
  	g_object_unref (gjob);
 
 	gedit_print_job_save_config (GEDIT_PRINT_JOB (pjob));
-	
+
 	g_object_unref (pjob);
+
+	if (tab->priv->print_preview != NULL)
+	{
+		/* If we were printing while showing the print preview,
+		   see bug #352658 */
+		gtk_widget_destroy (tab->priv->print_preview);
+		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
+	}
 	
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 }
@@ -2089,6 +2127,14 @@ print_cancelled (GeditMessageArea *area,
 	g_object_unref (tab->priv->print_job);
 
 	set_message_area (tab, NULL); /* destroy the message area */
+
+	if (tab->priv->print_preview != NULL)
+	{
+		/* If we were printing while showing the print preview,
+		   see bug #352658 */        	
+		gtk_widget_destroy (tab->priv->print_preview);
+		g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINTING);
+	}
 	
 	gedit_tab_set_state (tab, GEDIT_TAB_STATE_NORMAL);
 }
