@@ -25,7 +25,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-window.c,v 1.41 2006/10/14 10:32:57 pborelli Exp $
+ * $Id: gedit-window.c,v 1.47 2006/12/10 17:31:47 pborelli Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -204,7 +204,6 @@ static void
 gedit_window_destroy (GtkObject *object)
 {
 	GeditWindow *window;
-	gint pane_page;
 
 	window = GEDIT_WINDOW (object);
 
@@ -256,29 +255,34 @@ gedit_window_configure_event (GtkWidget         *widget,
  * GtkWindow catches keybindings for the menu items _before_ passing them to
  * the focused widget. This is unfortunate and means that pressing ctrl+V
  * in an entry on a panel ends up pasting text in the TextView.
- * Here we force events to be first  passed to the focused widget and then
- * we chain up the default handler... this is the opposite of Gtk default
- * behavior so we need to keep an eye open to see if anything breaks.
+ * Here we override GtkWindow's handler to do the same things that it
+ * does, but in the opposite order and then we chain up to the grand
+ * parent handler, skipping gtk_window_key_press_event.
  */
 static gboolean
 gedit_window_key_press_event (GtkWidget   *widget,
 			      GdkEventKey *event)
 {
-	GtkWidget *focused_widget;
+	static gpointer grand_parent_class = NULL;
+	GtkWindow *window = GTK_WINDOW (widget);
 	gboolean handled = FALSE;
 
-	focused_widget = gtk_window_get_focus (GTK_WINDOW (widget));
+	if (grand_parent_class == NULL)
+		grand_parent_class = g_type_class_peek_parent (gedit_window_parent_class);
 
-	if (focused_widget)
-	{
-		handled = gtk_widget_event (focused_widget,
-					    (GdkEvent*) event);
-	}
+	/* handle focus widget key events */
+	if (!handled)
+		handled = gtk_window_propagate_key_event (window, event);
 
-	if (handled)
-		return TRUE;
-	else
-		return GTK_WIDGET_CLASS (gedit_window_parent_class)->key_press_event (widget, event);
+	/* handle mnemonics and accelerators */
+	if (!handled)
+		handled = gtk_window_activate_key (window, event);
+
+	/* Chain up, invokes binding set */
+	if (!handled)
+		handled = GTK_WIDGET_CLASS (grand_parent_class)->key_press_event (widget, event);
+
+	return handled;
 }
 
 static void
@@ -574,6 +578,69 @@ update_next_prev_doc_sensitivity_per_window (GeditWindow *window)
 	
 }
 
+static void
+received_clipboard_contents (GtkClipboard     *clipboard,
+			     GtkSelectionData *selection_data,
+			     GeditWindow      *window)
+{
+	gboolean sens;
+	GtkAction *action;
+
+	/* getting clipboard contents is async, so we need to
+	 * get the current tab and its state */
+
+	if (window->priv->active_tab != NULL)
+	{
+		GeditTabState state;
+		gboolean state_normal;
+
+		state = gedit_tab_get_state (window->priv->active_tab);
+		state_normal = (state == GEDIT_TAB_STATE_NORMAL);
+
+		sens = state_normal &&
+		       gtk_selection_data_targets_include_text (selection_data);
+	}
+	else
+	{
+		sens = FALSE;
+	}
+
+	action = gtk_action_group_get_action (window->priv->action_group,
+					      "EditPaste");
+
+	gtk_action_set_sensitive (action, sens);
+
+	g_object_unref (window);
+}
+
+static void
+set_paste_sensitivity_according_to_clipboard (GeditWindow  *window,
+					      GtkClipboard *clipboard)
+{
+	GdkDisplay *display;
+
+	display = gtk_clipboard_get_display (clipboard);
+
+	if (gdk_display_supports_selection_notification (display))
+	{
+		gtk_clipboard_request_contents (clipboard,
+						gdk_atom_intern ("TARGETS", FALSE),
+						(GtkClipboardReceivedFunc) received_clipboard_contents,
+						g_object_ref (window));
+	}
+	else
+	{
+		GtkAction *action;
+
+		action = gtk_action_group_get_action (window->priv->action_group,
+						      "EditPaste");
+
+		/* XFIXES extension not availbale, make
+		 * Paste always sensitive */
+		gtk_action_set_sensitive (action, TRUE);
+	}
+}
+
 static void
 set_sensitivity_according_to_tab (GeditWindow *window,
 				  GeditTab    *tab)
@@ -584,6 +651,7 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	gboolean       b;
 	gboolean       state_normal;
 	GeditTabState  state;
+	GtkClipboard  *clipboard;
 	GeditLockdownMask lockdown;
 
 	g_return_if_fail (GEDIT_TAB (tab));
@@ -598,6 +666,9 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 	view = gedit_tab_get_view (tab);
 	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer (GTK_TEXT_VIEW (view)));
 
+	clipboard = gtk_widget_get_clipboard (GTK_WIDGET (window),
+					      GDK_SELECTION_CLIPBOARD);
+
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "FileSave");
 	gtk_action_set_sensitive (action,
@@ -667,8 +738,15 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 				  
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "EditPaste");
-	gtk_action_set_sensitive (action,
-				  state_normal);
+	if (state_normal)
+	{
+		set_paste_sensitivity_according_to_clipboard (window,
+							      clipboard);
+	}
+	else
+	{
+		gtk_action_set_sensitive (action, FALSE);
+	}
 
 	action = gtk_action_group_get_action (window->priv->action_group,
 					      "EditDelete");
@@ -682,7 +760,7 @@ set_sensitivity_according_to_tab (GeditWindow *window,
 				  state_normal);
 
 	action = gtk_action_group_get_action (window->priv->action_group,
-					      "SearchInteractiveSearch");
+					      "SearchIncrementalSearch");
 	gtk_action_set_sensitive (action,
 				  state_normal);
 
@@ -830,8 +908,10 @@ create_language_menu_item (GtkSourceLanguage *lang,
 				       NULL,
 				       index);
 
-	gtk_action_group_add_action (window->priv->languages_action_group,
-				     GTK_ACTION (action));
+	/* Action is added with a NULL accel to make the accel overridable */
+	gtk_action_group_add_action_with_accel (window->priv->languages_action_group,
+						GTK_ACTION (action),
+						NULL);
 	g_object_unref (action);
 
 	/* add the action to the same radio group of the "Normal" action */
@@ -1061,7 +1141,7 @@ recent_manager_changed (GtkRecentManager *manager,
 	update_recent_files_menu (window);
 }
 
-#define TIP_MAX_URI_LEN 100
+
 
 /*
  * Manually construct the inline recents list in the File menu.
@@ -1073,6 +1153,7 @@ update_recent_files_menu (GeditWindow *window)
 	GeditWindowPrivate *p = window->priv;
 	gint max_recents;
 	GList *actions, *l, *items;
+	GList *filtered_items = NULL;
 	gint i;
 
 	gedit_debug (DEBUG_WINDOW);
@@ -1100,17 +1181,30 @@ update_recent_files_menu (GeditWindow *window)
 
 	items = gtk_recent_manager_get_items (p->recent_manager);
 
-	items = g_list_sort (items, (GCompareFunc) sort_recents_mru);
+	/* filter */
+	for (l = items; l != NULL; l = l->next)
+	{
+		GtkRecentInfo *info = l->data;
+
+		if (!gtk_recent_info_has_group (info, "gedit"))
+			continue;
+
+		filtered_items = g_list_prepend (filtered_items, info);
+	}
+
+	/* sort */
+	filtered_items = g_list_sort (filtered_items,
+				      (GCompareFunc) sort_recents_mru);
 
 	i = 0;
-	for (l = items; l != NULL; l = l->next)
+	for (l = filtered_items; l != NULL; l = l->next)
 	{
 		gchar *action_name;
 		const gchar *display_name;
 		gchar *escaped;
 		gchar *label;
 		gchar *uri;
-		gchar *trunc_uri;
+		gchar *ruri;
 		gchar *tip;
 		GtkAction *action;
 		GtkRecentInfo *info = l->data;
@@ -1119,10 +1213,6 @@ update_recent_files_menu (GeditWindow *window)
 		if (i >= max_recents)
 			break;
 
-		/* filter */
-		if (!gtk_recent_info_has_group (info, "gedit"))
-			continue;
-
 		i++;
 
 		action_name = g_strdup_printf ("recent-info-%d", i);
@@ -1142,13 +1232,12 @@ update_recent_files_menu (GeditWindow *window)
 		/* gtk_recent_info_get_uri_display (info) is buggy and
 		 * works only for local files */
 		uri = gedit_utils_format_uri_for_display (gtk_recent_info_get_uri (info));
-		trunc_uri = gedit_utils_str_middle_truncate (uri,
-							     TIP_MAX_URI_LEN);
+		ruri = gedit_utils_replace_home_dir_with_tilde (uri);
 		g_free (uri);
 
 		/* Translators: %s is a URI */
-		tip = g_strdup_printf (_("Open '%s'"), trunc_uri);
-		g_free (trunc_uri);
+		tip = g_strdup_printf (_("Open '%s'"), ruri);
+		g_free (ruri);
 
 		action = gtk_action_new (action_name,
 					 label,
@@ -1181,12 +1270,12 @@ update_recent_files_menu (GeditWindow *window)
 		g_free (tip);
 	}
 
+	g_list_free (filtered_items);
+
 	g_list_foreach (items, (GFunc) gtk_recent_info_unref, NULL);
 	g_list_free (items);
 }
 
-#undef TIP_MAX_URI_LEN
-
 static void
 set_non_homogeneus (GtkWidget *widget, gpointer data)
 {
@@ -1439,6 +1528,27 @@ documents_list_menu_activate (GtkToggleAction *action,
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook), n);
 }
 
+static gchar *
+get_menu_tip_for_tab (GeditTab *tab)
+{
+	GeditDocument *doc;
+	gchar *uri;
+	gchar *ruri;
+	gchar *tip;
+
+	doc = gedit_tab_get_document (tab);
+
+	uri = gedit_document_get_uri_for_display (doc);
+	ruri = gedit_utils_replace_home_dir_with_tilde (uri);
+	g_free (uri);
+
+	/* Translators: %s is a URI */
+	tip =  g_strdup_printf (_("Activate '%s'"), ruri);
+	g_free (ruri);
+	
+	return tip;
+}
+
 static void
 update_documents_list_menu (GeditWindow *window)
 {
@@ -1493,7 +1603,7 @@ update_documents_list_menu (GeditWindow *window)
 		action_name = g_strdup_printf ("Tab_%d", i);
 		tab_name = _gedit_tab_get_name (GEDIT_TAB (tab));
 		name = gedit_utils_escape_underscores (tab_name, -1);
-		tip =  g_strdup_printf (_("Activate %s"), tab_name);
+		tip =  get_menu_tip_for_tab (GEDIT_TAB (tab));
 
 		/* alt + 1, 2, 3... 0 to switch to the first ten tabs */
 		accel = (i < 10) ? g_strdup_printf ("<alt>%d", (i + 1) % 10) : NULL;
@@ -2143,7 +2253,7 @@ sync_name (GeditTab    *tab,
 
 	tab_name = _gedit_tab_get_name (tab);
 	escaped_name = gedit_utils_escape_underscores (tab_name, -1);
-	tip =  g_strdup_printf (_("Activate %s"), tab_name);
+	tip =  get_menu_tip_for_tab (tab);
 
 	g_object_set (action, "label", escaped_name, NULL);
 	g_object_set (action, "tooltip", tip, NULL);
@@ -3059,6 +3169,44 @@ init_panels_visibility (GeditWindow *window)
 			  window);
 }
 
+static void
+clipboard_owner_change (GtkClipboard        *clipboard,
+			GdkEventOwnerChange *event,
+			GeditWindow         *window)
+{
+	set_paste_sensitivity_according_to_clipboard (window,
+						      clipboard);
+}
+
+static void
+window_realized (GtkWidget *window,
+		 gpointer  *data)
+{
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_widget_get_clipboard (window,
+					      GDK_SELECTION_CLIPBOARD);
+
+	g_signal_connect (clipboard,
+			  "owner_change",
+			  G_CALLBACK (clipboard_owner_change),
+			  window);
+}
+
+static void
+window_unrealized (GtkWidget *window,
+		   gpointer  *data)
+{
+	GtkClipboard *clipboard;
+
+	clipboard = gtk_widget_get_clipboard (window,
+					      GDK_SELECTION_CLIPBOARD);
+
+	g_signal_handlers_disconnect_by_func (clipboard,
+					      G_CALLBACK (clipboard_owner_change),
+					      window);
+}
+
 static void
 gedit_window_init (GeditWindow *window)
 {
@@ -3186,6 +3334,17 @@ gedit_window_init (GeditWindow *window)
 	                  G_CALLBACK (drag_data_received_cb), 
 	                  NULL);
 
+	/* we can get the clipboard only after the widget
+	 * is realized */
+	g_signal_connect (window,
+			  "realize",
+			  G_CALLBACK (window_realized),
+			  NULL);
+	g_signal_connect (window,
+			  "unrealize",
+			  G_CALLBACK (window_unrealized),
+			  NULL);
+
 	gedit_debug_message (DEBUG_WINDOW, "Update plugins ui");
 	gedit_plugins_engine_update_plugins_ui (window, TRUE);
 
