@@ -206,12 +206,19 @@ init_special_directories (GeditFileBookmarksStore * model)
 	local = g_build_filename (path, "Documents", NULL);
 	uri = gnome_vfs_uri_new (local);
 
-	if (gnome_vfs_uri_exists (uri))
+	if (gnome_vfs_uri_is_local (uri)) {
+		if (gnome_vfs_uri_exists (uri))
+			add_uri (model, uri, NULL,
+				 GEDIT_FILE_BOOKMARKS_STORE_IS_DOCUMENTS |
+				 GEDIT_FILE_BOOKMARKS_STORE_IS_SPECIAL_DIR, NULL);
+		else
+			gnome_vfs_uri_unref (uri);
+	} else {
 		add_uri (model, uri, NULL,
 			 GEDIT_FILE_BOOKMARKS_STORE_IS_DOCUMENTS |
 			 GEDIT_FILE_BOOKMARKS_STORE_IS_SPECIAL_DIR, NULL);
-	else
-		gnome_vfs_uri_unref (uri);
+	}
+		
 
 	g_free (local);
 }
@@ -399,13 +406,11 @@ init_bookmarks (GeditFileBookmarksStore * model)
 
 		/* Add a watch */
 		if (model->priv->bookmarks_monitor == NULL) {
-			gnome_vfs_monitor_add (&
-					       (model->priv->
-						bookmarks_monitor),
+			gnome_vfs_monitor_add (&model->priv->bookmarks_monitor,
 					       bookmarks,
 					       GNOME_VFS_MONITOR_FILE,
-					       (GnomeVFSMonitorCallback)
-on_bookmarks_file_changed, model);
+					       (GnomeVFSMonitorCallback)on_bookmarks_file_changed, 
+					       model);
 		}
 	} else {
 		/* The bookmarks file doesn't exist (which is perfectly fine) */
