@@ -111,41 +111,41 @@ static GtkTreeModelFlags gedit_file_browser_store_get_flags (GtkTreeModel * tree
 static gint gedit_file_browser_store_get_n_columns          (GtkTreeModel * tree_model);
 static GType gedit_file_browser_store_get_column_type       (GtkTreeModel * tree_model,
 							     gint index);
-
 static gboolean gedit_file_browser_store_get_iter           (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter,
 							     GtkTreePath * path);
-
 static GtkTreePath *gedit_file_browser_store_get_path       (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter);
-
 static void gedit_file_browser_store_get_value              (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter,
 							     gint column,
 							     GValue * value);
-
 static gboolean gedit_file_browser_store_iter_next          (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter);
-
 static gboolean gedit_file_browser_store_iter_children      (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter,
 							     GtkTreeIter * parent);
-
 static gboolean gedit_file_browser_store_iter_has_child     (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter);
-
 static gint gedit_file_browser_store_iter_n_children        (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter);
-
 static gboolean gedit_file_browser_store_iter_nth_child     (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter,
 							     GtkTreeIter * parent, 
 							     gint n);
-
 static gboolean gedit_file_browser_store_iter_parent        (GtkTreeModel * tree_model,
 							     GtkTreeIter * iter,
 							     GtkTreeIter * child);
 
+static void gedit_file_browser_store_drag_source_init       (GtkTreeDragSourceIface * iface);
+static gboolean gedit_file_browser_store_row_draggable      (GtkTreeDragSource * drag_source,
+							     GtkTreePath       * path);
+static gboolean gedit_file_browser_store_drag_data_delete   (GtkTreeDragSource * drag_source,
+							     GtkTreePath       * path);
+static gboolean gedit_file_browser_store_drag_data_get      (GtkTreeDragSource * drag_source,
+							     GtkTreePath       * path,
+							     GtkSelectionData  * selection_data);
+
 static void file_browser_node_free                          (GeditFileBrowserStore * model,
 							     FileBrowserNode * node);
 static void model_add_node                                  (GeditFileBrowserStore * model,
@@ -170,7 +170,9 @@ static void on_directory_monitor_event                      (GnomeVFSMonitorHand
 GEDIT_PLUGIN_DEFINE_TYPE_WITH_CODE (GeditFileBrowserStore, gedit_file_browser_store,
 			G_TYPE_OBJECT,
 			G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_MODEL,
-					       gedit_file_browser_store_iface_init))
+					       gedit_file_browser_store_iface_init)
+			G_IMPLEMENT_INTERFACE (GTK_TYPE_TREE_DRAG_SOURCE,
+					       gedit_file_browser_store_drag_source_init))
 
 /* Properties */
 enum {
@@ -262,6 +264,7 @@ gedit_file_browser_store_set_property (GObject      *object,
 		case PROP_FILTER_MODE:
 			gedit_file_browser_store_set_filter_mode (obj,
 			                                          g_value_get_flags (value));
+			break;
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 			break;
@@ -345,6 +348,14 @@ gedit_file_browser_store_iface_init (GtkTreeModelIface * iface)
 	iface->iter_parent = gedit_file_browser_store_iter_parent;
 }
 
+static void
+gedit_file_browser_store_drag_source_init (GtkTreeDragSourceIface * iface)
+{
+	iface->row_draggable = gedit_file_browser_store_row_draggable;
+	iface->drag_data_delete = gedit_file_browser_store_drag_data_delete;
+	iface->drag_data_get = gedit_file_browser_store_drag_data_get;
+}
+
 static void
 gedit_file_browser_store_init (GeditFileBrowserStore * obj)
 {
@@ -498,7 +509,7 @@ gedit_file_browser_store_get_path_real (GeditFileBrowserStore * model,
 	FileBrowserNode *check;
 	GSList *item;
 	gint num = 0;
-
+	
 	while (node != model->priv->virtual_root) {
 		if (node->parent == NULL) {
 			gtk_tree_path_free (path);
@@ -808,6 +819,63 @@ gedit_file_browser_store_iter_parent (GtkTreeModel * tree_model,
 	return TRUE;
 }
 
+static gboolean
+gedit_file_browser_store_row_draggable (GtkTreeDragSource * drag_source,
+					GtkTreePath       * path)
+{
+	GtkTreeIter iter;
+	GeditFileBrowserStoreFlag flags;
+
+	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_source),
+				      &iter, path))
+	{
+		return FALSE;
+	}
+
+	gtk_tree_model_get (GTK_TREE_MODEL (drag_source), &iter,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_FLAGS, &flags,
+			    -1);
+
+	return !FILE_IS_DUMMY(flags);
+}
+
+static gboolean
+gedit_file_browser_store_drag_data_delete (GtkTreeDragSource * drag_source,
+					   GtkTreePath       * path)
+{
+	return FALSE;
+}
+
+static gboolean
+gedit_file_browser_store_drag_data_get (GtkTreeDragSource * drag_source,
+					GtkTreePath       * path,
+					GtkSelectionData  * selection_data)
+{
+	GtkTreeIter iter;
+	gchar *uri;
+	gchar *uris[2] = {0, };
+	gboolean ret;
+
+	if (!gtk_tree_model_get_iter (GTK_TREE_MODEL (drag_source),
+				      &iter, path))
+	{
+		return FALSE;
+	}
+
+	gtk_tree_model_get (GTK_TREE_MODEL (drag_source), &iter,
+			    GEDIT_FILE_BROWSER_STORE_COLUMN_URI, &uri,
+			    -1);
+
+	g_assert (uri);
+
+	uris[0] = uri;
+	ret = gtk_selection_data_set_uris (selection_data, uris);
+
+	g_free (uri);
+
+	return ret;
+}
+
 #define FILTER_HIDDEN(mode) (mode & GEDIT_FILE_BROWSER_STORE_FILTER_MODE_HIDE_HIDDEN)
 #define FILTER_BINARY(mode) (mode & GEDIT_FILE_BROWSER_STORE_FILTER_MODE_HIDE_BINARY)
 
@@ -855,12 +923,32 @@ model_node_update_visibility (GeditFileBrowserStore * model,
 	}
 }
 
+static gint
+collate_nodes (FileBrowserNode * node1, FileBrowserNode * node2)
+{
+	if (node1->name == NULL)
+		return -1;
+	else if (node2->name == NULL)
+		return 1;
+	else {
+		gchar *k1, *k2;
+		gint result;
+
+		k1 = g_utf8_collate_key_for_filename (node1->name, -1);
+		k2 = g_utf8_collate_key_for_filename (node2->name, -1);
+
+		result = strcmp (k1, k2);
+
+		g_free (k1);
+		g_free (k2);
+
+		return result;
+	}
+}
+
 static gint
 model_sort_default (FileBrowserNode * node1, FileBrowserNode * node2)
 {
-	gchar *case1;
-	gchar *case2;
-	gint result;
 	gint f1;
 	gint f2;
 
@@ -895,21 +983,7 @@ model_sort_default (FileBrowserNode * node1, FileBrowserNode * node2)
 			return 1;
 	}
 
-	if (node1->name == NULL)
-		return -1;
-	else if (node2->name == NULL)
-		return 1;
-	else {
-		case1 = g_utf8_casefold (node1->name, -1);
-		case2 = g_utf8_casefold (node2->name, -1);
-
-		result = g_utf8_collate (case1, case2);
-
-		g_free (case1);
-		g_free (case2);
-
-		return result;
-	}
+	return collate_nodes (node1, node2);
 }
 
 static void
@@ -1228,10 +1302,6 @@ file_browser_node_free (GeditFileBrowserStore * model,
 	g_free (node);
 }
 
-/* Should only be called for removing nodes that are visible in the model 
- * ======================================================================
- */
-
 /**
  * model_remove_node_children:
  * @model: the #GeditFileBrowserStore
@@ -1261,6 +1331,15 @@ model_remove_node_children (GeditFileBrowserStore * model,
 	if (dir->children == NULL)
 		return;
 
+	if (!model_node_visibility (model, node)) {
+		// Node is invisible and therefore the children can just
+		// be freed
+		if (free_nodes)
+			file_browser_node_free_children (model, node);
+		
+		return;
+	}
+
 	if (path == NULL)
 		path_child =
 		    gedit_file_browser_store_get_path_real (model, node);
@@ -1268,6 +1347,7 @@ model_remove_node_children (GeditFileBrowserStore * model,
 		path_child = gtk_tree_path_copy (path);
 
 	gtk_tree_path_down (path_child);
+
 	list = g_slist_copy (dir->children);
 
 	for (item = list; item; item = item->next)
